/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tqlab.sense.servlet.view.velocity;

import java.io.File;
import java.io.IOException;

import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;
import org.springframework.core.io.Resource;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.view.velocity.VelocityConfigurer;

import com.tqlab.sense.Activator;

/**
 * @author John Lee
 * 
 */
public class SenseVelocityConfigurer extends VelocityConfigurer {

	protected void initVelocityResourceLoader(VelocityEngine velocityEngine,
			String resourceLoaderPath) {
		if (isPreferFileSystemAccess()) {
			// Try to load via the file system, fall back to
			// SpringResourceLoader
			// (for hot detection of template changes, if possible).
			try {
				StringBuilder resolvedPath = new StringBuilder();
				String[] paths = StringUtils
						.commaDelimitedListToStringArray(resourceLoaderPath);
				String dir = System.getProperty(Activator.TEMPLATE_DIR_KEY);
				if (null == dir) {
					dir = "";
				}
				dir = ClassLoader.getSystemResource("").toExternalForm() + dir;
				
				dir = dir.replace("%20", " ");

				for (int i = 0; i < paths.length; i++) {
					String path = dir + paths[i];
					// ClassLoader.getSystemResource(path);
					Resource resource = getResourceLoader().getResource(path);
					File file = resource.getFile(); // will fail if not
													// resolvable in the file
													// system
					if (logger.isDebugEnabled()) {
						logger.debug("Resource loader path [" + path
								+ "] resolved to file ["
								+ file.getAbsolutePath() + "]");
					}
					resolvedPath.append(file.getAbsolutePath());
					if (i < paths.length - 1) {
						resolvedPath.append(',');
					}
				}
				velocityEngine.setProperty(RuntimeConstants.RESOURCE_LOADER,
						"file");
				velocityEngine.setProperty(
						RuntimeConstants.FILE_RESOURCE_LOADER_CACHE, "true");
				velocityEngine.setProperty(
						RuntimeConstants.FILE_RESOURCE_LOADER_PATH,
						resolvedPath.toString());
			} catch (IOException ex) {
				if (logger.isDebugEnabled()) {
					logger.debug(
							"Cannot resolve resource loader path ["
									+ resourceLoaderPath
									+ "] to [java.io.File]: using SpringResourceLoader",
							ex);
				}
				initSpringResourceLoader(velocityEngine, resourceLoaderPath);
			}
		} else {
			// Always load via SpringResourceLoader
			// (without hot detection of template changes).
			if (logger.isDebugEnabled()) {
				logger.debug("File system access not preferred: using SpringResourceLoader");
			}
			initSpringResourceLoader(velocityEngine, resourceLoaderPath);
		}
	}
}
