/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tqlab.sense.servlet.view.velocity;

import java.util.Locale;

import org.springframework.util.StringUtils;
import org.springframework.web.servlet.view.AbstractUrlBasedView;
import org.springframework.web.servlet.view.velocity.VelocityViewResolver;

import com.tqlab.sense.context.WebApp;

/**
 * @author John Lee
 */
public class SenseVelocityViewResolver extends VelocityViewResolver {

	private String layoutUrl;
	private String layoutKey;
	private String screenContentKey;

	protected AbstractUrlBasedView buildView(String viewName) throws Exception {
		SenseVelocityLayoutView view = (SenseVelocityLayoutView) super
				.buildView(viewName);

		String typePrefix = "";
		// Use not-null checks to preserve VelocityLayoutView's defaults.
		if (StringUtils.hasText(viewName) && viewName.startsWith("tile:")) {
			viewName = viewName.substring("tile:".length());
			view.setLayout(false);
			typePrefix = "tile";
		} else if (StringUtils.hasText(viewName)
				&& viewName.startsWith("template:")) {
			viewName = viewName.substring("template:".length());
			view.setLayout(false);
			typePrefix = "template";
		} else if (StringUtils.hasText(viewName)
				&& viewName.startsWith("taglib:")) {
			viewName = viewName.substring("taglib:".length());
			view.setLayout(false);
			typePrefix = "taglib";
		} else {
			view.setLayoutUrl(WebApp.getCurrentSenceServletContext()
					.getModual() + "/layout/" + "default.vm");
			view.setLayout(true);
		}
		if (this.layoutUrl != null) {
			view.setLayoutUrl(this.layoutUrl);
		}
		if (this.layoutKey != null) {
			view.setLayoutKey(this.layoutKey);
		}
		if (this.screenContentKey != null) {
			view.setScreenContentKey(this.screenContentKey);
		}

		String url = "";
		if (typePrefix.equals("template")) {
			url = viewName;
		} else {
			url = getPrefix()
					+ WebApp.getCurrentSenceServletContext().getModual()
					+ "/"
					+ (StringUtils.hasText(typePrefix) ? (typePrefix + "/")
							: "screen/") + viewName + getSuffix();
		}

		view.setUrl(url);
		return view;
	}

	/**
	 * Return the cache key for the given view name and the given locale.
	 * <p>
	 * Default is a String consisting of view name and locale suffix. Can be
	 * overridden in subclasses.
	 * <p>
	 * Needs to respect the locale in general, as a different locale can lead to
	 * a different view resource.
	 */
	protected Object getCacheKey(String viewName, Locale locale) {
		return WebApp.getCurrentSenceServletContext().getModual() + "_"
				+ viewName + "_" + locale;
	}

	@SuppressWarnings("rawtypes")
	@Override
	protected Class requiredViewClass() {
		return SenseVelocityLayoutView.class;
	}

	public String getLayoutUrl() {
		return layoutUrl;
	}

	public void setLayoutUrl(String layoutUrl) {
		this.layoutUrl = layoutUrl;
	}

	public String getLayoutKey() {
		return layoutKey;
	}

	public void setLayoutKey(String layoutKey) {
		this.layoutKey = layoutKey;
	}

	public String getScreenContentKey() {
		return screenContentKey;
	}

	public void setScreenContentKey(String screenContentKey) {
		this.screenContentKey = screenContentKey;
	}
}
