/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tqlab.sense.servlet.view.velocity.tools;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.springframework.util.ClassUtils;
import org.springframework.util.ReflectionUtils;

/**
 * @author John Lee
 * 
 */
public class ToolBoxImpl implements ToolBox {

	public static final String TOOL_BOX_CONTEXT_NAME = "toolContext";
	private Map<String, Object> toolBoxMap;

	public void setToolBoxMap(Map<String, Object> toolBoxMap) {
		this.toolBoxMap = toolBoxMap;
	}

	public Map<String, Object> getToolsContext(HttpServletRequest request) {
		// Get cache
		@SuppressWarnings("unchecked")
		Map<String, Object> toolContext = (Map<String, Object>) request
				.getAttribute(TOOL_BOX_CONTEXT_NAME);
		if (null == toolContext) {
			toolContext = new HashMap<String, Object>();
			if (toolBoxMap != null) {
				for (String key : toolBoxMap.keySet()) {
					Object tool = toolBoxMap.get(key);
					Method initMethod = ClassUtils.getMethodIfAvailable(
							tool.getClass(), "init", new Class[] {
									Context.class, HttpServletRequest.class });
					if (initMethod != null) {
						initMethod.setAccessible(true);
						ReflectionUtils.invokeMethod(initMethod, tool,
								new Object[] {
										new VelocityContext(toolContext),
										request });
					} else {
						initMethod = ClassUtils.getMethodIfAvailable(
								tool.getClass(), "init",
								new Class[] { Context.class });
						if (initMethod != null) {
							initMethod.setAccessible(true);
							ReflectionUtils.invokeMethod(initMethod, tool,
									new Object[] { new VelocityContext(
											toolContext) });
						}
					}
					toolContext.put(key, tool);
				}
			}
			request.setAttribute(TOOL_BOX_CONTEXT_NAME, toolContext);
		}
		return toolContext;
	}

}
